use std::process::Command;
use std::time::Duration;
use std::time::SystemTime;

use crate::db::DbHandle;
use crate::db::DbLocalAction;

pub(crate) const ACTION_BUILD: &str = "build";
const ACTION_BUILD_INTERVAL: u64 = 1209600;
const ACTION_BUILD_JOBS: u64 = 1;

pub(crate) fn build(
    db_conn: &mut DbHandle,
    action: &DbLocalAction,
) -> Result<(), Box<dyn std::error::Error>> {
    println!(
        "Performing build for jail {} tree {} set {}.",
        action.jail_name.as_str(),
        action.tree.as_str(),
        action.set_name.as_str()
    );
    Command::new("poudriere")
        .arg("pkgclean")
        .arg("-v")
        .arg("-y")
        .arg("-j")
        .arg(action.jail_name.as_str())
        .arg("-p")
        .arg(action.tree.as_str())
        .arg("-z")
        .arg(action.set_name.as_str())
        .arg("-f")
        .arg(format!(
            "/usr/local/etc/poudriere.d/{}-{}-{}-pkglist",
            action.jail_name.as_str(),
            action.tree.as_str(),
            action.set_name.as_str()
        ))
        .status()?
        .exit_ok()?;
    let status = Command::new("poudriere")
        .arg("bulk")
        .arg("-J")
        .arg(ACTION_BUILD_JOBS.to_string())
        .arg("-j")
        .arg(action.jail_name.as_str())
        .arg("-p")
        .arg(action.tree.as_str())
        .arg("-z")
        .arg(action.set_name.as_str())
        .arg("-f")
        .arg(format!(
            "/usr/local/etc/poudriere.d/{}-{}-{}-pkglist",
            action.jail_name.as_str(),
            action.tree.as_str(),
            action.set_name.as_str()
        ))
        .status()?;
    if !status.success() {
        println!(
            "Build failed for jail {} tree {} set {}.",
            action.jail_name.as_str(),
            action.tree.as_str(),
            action.set_name.as_str()
        );
    }
    let next_run = (SystemTime::now() + Duration::from_secs(ACTION_BUILD_INTERVAL))
        .duration_since(SystemTime::UNIX_EPOCH)?
        .as_secs();

    let tx = db_conn.conn.transaction()?;
    tx.execute(
        "UPDATE local_action SET next_run=$1 WHERE name=$2 AND jail_name=$3 AND tree=$4 AND set_name=$5",
        (next_run, ACTION_BUILD, action.jail_name.as_str(), action.tree.as_str(), action.set_name.as_str()),
    )?;
    tx.commit()?;
    Ok(())
}
